@@ -4,7 +4,7 @@
 # may not use this file except in compliance with the License. A copy of
 # the License is located at
 #
-# http://aws.amazon.com/apache2.0/
+# https://aws.amazon.com/apache2.0/
 #
 # or in the "license" file accompanying this file. This file is
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
@@ -15,10 +15,12 @@ import copy
 import logging
 
 from botocore import xform_name
+from botocore.utils import merge_dicts
 
 from .action import BatchAction
 from .params import create_request_parameters
 from .response import ResourceHandler
+from ..docs import docstring
 
 
 logger = logging.getLogger(__name__)
@@ -47,14 +49,14 @@ class ResourceCollection(object):
         self._py_operation_name = xform_name(
             model.request.operation)
         self._handler = handler
-        self._params = kwargs
+        self._params = copy.deepcopy(kwargs)
 
     def __repr__(self):
         return '{0}({1}, {2})'.format(
             self.__class__.__name__,
             self._parent,
             '{0}.{1}'.format(
-                self._parent.meta['service_name'],
+                self._parent.meta.service_name,
                 self._model.resource.type
             )
         )
@@ -106,7 +108,7 @@ class ResourceCollection(object):
         :return: A clone of this resource collection
         """
         params = copy.deepcopy(self._params)
-        params.update(kwargs)
+        merge_dicts(params, kwargs, append_lists=True)
         clone = self.__class__(self._model, self._parent,
                                self._handler, **params)
         return clone
@@ -131,14 +133,13 @@ class ResourceCollection(object):
         :rtype: list(:py:class:`~boto3.resources.base.ServiceResource`)
         :return: List of resource instances
         """
-        client = self._parent.meta['client']
+        client = self._parent.meta.client
         cleaned_params = self._params.copy()
         limit = cleaned_params.pop('limit', None)
         page_size = cleaned_params.pop('page_size', None)
-
         params = create_request_parameters(
             self._parent, self._model.request)
-        params.update(cleaned_params)
+        merge_dicts(params, cleaned_params, append_lists=True)
 
         # Is this a paginated operation? If so, we need to get an
         # iterator for the various pages. If not, then we simply
@@ -146,16 +147,17 @@ class ResourceCollection(object):
         # page in a list. For non-paginated results, we just ignore
         # the page size parameter.
         if client.can_paginate(self._py_operation_name):
-            logger.info('Calling paginated %s:%s with %r',
-                        self._parent.meta['service_name'],
-                        self._py_operation_name, params)
+            logger.debug('Calling paginated %s:%s with %r',
+                         self._parent.meta.service_name,
+                         self._py_operation_name, params)
             paginator = client.get_paginator(self._py_operation_name)
             pages = paginator.paginate(
-                max_items=limit, page_size=page_size, **params)
+                PaginationConfig={
+                    'MaxItems': limit, 'PageSize': page_size}, **params)
         else:
-            logger.info('Calling %s:%s with %r',
-                        self._parent.meta['service_name'],
-                        self._py_operation_name, params)
+            logger.debug('Calling %s:%s with %r',
+                         self._parent.meta.service_name,
+                         self._py_operation_name, params)
             pages = [getattr(client, self._py_operation_name)(**params)]
 
         # Now that we have a page iterator or single page of results
@@ -178,7 +180,7 @@ class ResourceCollection(object):
             if limit is not None and count >= limit:
                 break
 
-    def all(self, limit=None, page_size=None):
+    def all(self):
         """
         Get all items from the collection, optionally with a custom
         page size and item count limit.
@@ -196,14 +198,8 @@ class ResourceCollection(object):
             >>> queues = list(sqs.queues.all())
             >>> len(queues)
             2
-
-        :type limit: int
-        :param limit: Return no more than this many items
-        :type page_size: int
-        :param page_size: Fetch this many items per request
-        :rtype: :py:class:`ResourceCollection`
         """
-        return self._clone(limit=limit, page_size=page_size)
+        return self._clone()
 
     def filter(self, **kwargs):
         """
@@ -292,36 +288,40 @@ class CollectionManager(object):
     See the :ref:`guide_collections` guide for a high-level overview
     of collections, including when remote service requests are performed.
 
-    :type model: :py:class:`~boto3.resources.model.Collection`
+    :type collection_model: :py:class:`~boto3.resources.model.Collection`
     :param model: Collection model
+
     :type parent: :py:class:`~boto3.resources.base.ServiceResource`
     :param parent: The collection's parent resource
+
     :type factory: :py:class:`~boto3.resources.factory.ResourceFactory`
     :param factory: The resource factory to create new resources
-    :type resource_defs: dict
-    :param resource_defs: Service resource definitions.
-    :type service_model: :ref:`botocore.model.ServiceModel`
-    :param service_model: The Botocore service model
+
+    :type service_context: :py:class:`~boto3.utils.ServiceContext`
+    :param service_context: Context about the AWS service
     """
     # The class to use when creating an iterator
     _collection_cls = ResourceCollection
 
-    def __init__(self, model, parent, factory, resource_defs,
-                 service_model):
-        self._model = model
+    def __init__(self, collection_model, parent, factory, service_context):
+        self._model = collection_model
         operation_name = self._model.request.operation
         self._parent = parent
 
-        search_path = model.path
-        self._handler = ResourceHandler(search_path, factory, resource_defs,
-            service_model, model.resource, operation_name)
+        search_path = collection_model.resource.path
+        self._handler = ResourceHandler(
+            search_path=search_path, factory=factory,
+            resource_model=collection_model.resource,
+            service_context=service_context,
+            operation_name=operation_name
+        )
 
     def __repr__(self):
         return '{0}({1}, {2})'.format(
             self.__class__.__name__,
             self._parent,
             '{0}.{1}'.format(
-                self._parent.meta['service_name'],
+                self._parent.meta.service_name,
                 self._model.resource.type
             )
         )
@@ -337,8 +337,8 @@ class CollectionManager(object):
                                     self._handler, **kwargs)
 
     # Set up some methods to proxy ResourceCollection methods
-    def all(self, limit=None, page_size=None):
-        return self.iterator(limit=limit, page_size=page_size)
+    def all(self):
+        return self.iterator()
     all.__doc__ = ResourceCollection.all.__doc__
 
     def filter(self, **kwargs):
@@ -365,8 +365,8 @@ class CollectionFactory(object):
     subclasses from a :py:class:`~boto3.resources.model.Collection`
     model. These subclasses include methods to perform batch operations.
     """
-    def load_from_definition(self, service_name, resource_name,
-                             collection_name, model, resource_defs):
+    def load_from_definition(self, resource_name, collection_model,
+                             service_context, event_emitter):
         """
         Loads a collection from a model, creating a new
         :py:class:`CollectionManager` subclass
@@ -375,50 +375,136 @@ class CollectionFactory(object):
         creates a new :py:class:`ResourceCollection` subclass which is used
         by the new manager class.
 
-        :type service_name: string
-        :param service_name: Name of the service to look up
         :type resource_name: string
         :param resource_name: Name of the resource to look up. For services,
                               this should match the ``service_name``.
-        :type model: dict
-        :param model: The collection definition.
-        :type resource_defs: dict
-        :param resource_defs: The service's resource definitions, used to load
-                              collection resources (e.g. ``sqs.Queue``).
-        :rtype: Subclass of
-                :py:class:`CollectionManager`
+
+        :type service_context: :py:class:`~boto3.utils.ServiceContext`
+        :param service_context: Context about the AWS service
+
+        :type event_emitter: :py:class:`~botocore.hooks.HierarchialEmitter`
+        :param event_emitter: An event emitter
+
+        :rtype: Subclass of :py:class:`CollectionManager`
         :return: The collection class.
         """
         attrs = {}
+        collection_name = collection_model.name
 
-        self._load_batch_actions(attrs, model)
+        # Create the batch actions for a collection
+        self._load_batch_actions(
+            attrs, resource_name, collection_model,
+            service_context.service_model, event_emitter)
+        # Add the documentation to the collection class's methods
+        self._load_documented_collection_methods(
+            attrs=attrs, resource_name=resource_name,
+            collection_model=collection_model,
+            service_model=service_context.service_model,
+            event_emitter=event_emitter,
+            base_class=ResourceCollection)
 
-        if service_name == resource_name:
+        if service_context.service_name == resource_name:
             cls_name = '{0}.{1}Collection'.format(
-                service_name, collection_name)
+                service_context.service_name, collection_name)
         else:
             cls_name = '{0}.{1}.{2}Collection'.format(
-                service_name, resource_name, collection_name)
+                service_context.service_name, resource_name, collection_name)
 
         collection_cls = type(str(cls_name), (ResourceCollection,),
                               attrs)
 
+        # Add the documentation to the collection manager's methods
+        self._load_documented_collection_methods(
+            attrs=attrs, resource_name=resource_name,
+            collection_model=collection_model,
+            service_model=service_context.service_model,
+            event_emitter=event_emitter,
+            base_class=CollectionManager)
         attrs['_collection_cls'] = collection_cls
         cls_name += 'Manager'
 
         return type(str(cls_name), (CollectionManager,), attrs)
 
-    def _load_batch_actions(self, attrs, model):
+    def _load_batch_actions(self, attrs, resource_name, collection_model,
+                            service_model, event_emitter):
         """
         Batch actions on the collection become methods on both
         the collection manager and iterators.
         """
-        for action_model in model.batch_actions:
+        for action_model in collection_model.batch_actions:
             snake_cased = xform_name(action_model.name)
             attrs[snake_cased] = self._create_batch_action(
-                snake_cased, action_model)
+                resource_name, snake_cased, action_model, collection_model,
+                service_model, event_emitter)
 
-    def _create_batch_action(factory_self, snake_cased, action_model):
+    def _load_documented_collection_methods(
+            factory_self, attrs, resource_name, collection_model,
+            service_model, event_emitter, base_class):
+        # The base class already has these methods defined. However
+        # the docstrings are generic and not based for a particular service
+        # or resource. So we override these methods by proxying to the
+        # base class's builtin method and adding a docstring
+        # that pertains to the resource.
+
+        # A collection's all() method.
+        def all(self):
+            return base_class.all(self)
+
+        all.__doc__ = docstring.CollectionMethodDocstring(
+            resource_name=resource_name,
+            action_name='all',
+            event_emitter=event_emitter,
+            collection_model=collection_model,
+            service_model=service_model,
+            include_signature=False
+        )
+        attrs['all'] = all
+
+        # The collection's filter() method.
+        def filter(self, **kwargs):
+            return base_class.filter(self, **kwargs)
+
+        filter.__doc__ = docstring.CollectionMethodDocstring(
+            resource_name=resource_name,
+            action_name='filter',
+            event_emitter=event_emitter,
+            collection_model=collection_model,
+            service_model=service_model,
+            include_signature=False
+        )
+        attrs['filter'] = filter
+
+        # The collection's limit method.
+        def limit(self, count):
+            return base_class.limit(self, count)
+
+        limit.__doc__ = docstring.CollectionMethodDocstring(
+            resource_name=resource_name,
+            action_name='limit',
+            event_emitter=event_emitter,
+            collection_model=collection_model,
+            service_model=service_model,
+            include_signature=False
+        )
+        attrs['limit'] = limit
+
+        # The collection's page_size method.
+        def page_size(self, count):
+            return base_class.page_size(self, count)
+
+        page_size.__doc__ = docstring.CollectionMethodDocstring(
+            resource_name=resource_name,
+            action_name='page_size',
+            event_emitter=event_emitter,
+            collection_model=collection_model,
+            service_model=service_model,
+            include_signature=False
+        )
+        attrs['page_size'] = page_size
+
+    def _create_batch_action(factory_self, resource_name, snake_cased,
+                             action_model, collection_model, service_model,
+                             event_emitter):
         """
         Creates a new method which makes a batch operation request
         to the underlying service API.
@@ -429,5 +515,12 @@ class CollectionFactory(object):
             return action(self, *args, **kwargs)
 
         batch_action.__name__ = str(snake_cased)
-        batch_action.__doc__ = 'TODO'
+        batch_action.__doc__ = docstring.BatchActionDocstring(
+            resource_name=resource_name,
+            event_emitter=event_emitter,
+            batch_action_model=action_model,
+            service_model=service_model,
+            collection_model=collection_model,
+            include_signature=False
+        )
         return batch_action
